/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.easymock.EasyMock;
import org.junit.Assert;
import org.junit.Rule;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlReaders;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlWriters;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlWriter;
import org.ow2.petals.jbi.servicedesc.endpoint.PetalsServiceEndpoint;
import org.ow2.petals.jbi.servicedesc.endpoint.PetalsServiceEndpoint.EndpointType;
import org.ow2.petals.microkernel.api.communication.sharedarea.exception.SharedAreaException;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration.DomainMode;
import org.ow2.petals.microkernel.api.util.ConfigurationServiceUtils;
import org.ow2.petals.registry_overlay.client.configuration.ConfigurationBuilder;
import org.ow2.petals.registry_overlay.junit.PetalsRegistryOverlayNode;
import org.ow2.petals.topology.generated.Topology;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.google.common.base.Optional;

/**
 * Abstract class for unit tests of {@link RegistryOverlayImpl}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class AbstractRegistryOverlayImplTest extends Assert {

    /**
     * {@link ConfigurationService#isSecurityTopologyPassphraseOk(String)} will return <code>true</code> for the
     * following topology pass-phrase.
     */
    protected final static String VALID_TOPOLOGY_PASSPHRASE = "my-valid-topology-passphrase";

    /**
     * {@link ConfigurationService#isSecurityTopologyPassphraseOk(String)} will return <code>false</code> for the
     * following topology pass-phrase.
     */
    protected final static String INVALID_TOPOLOGY_PASSPHRASE = "my-invalid-topology-passphrase";

    @Rule
    public final PetalsRegistryOverlayNode PETALS_REGISTRY_NODE = new PetalsRegistryOverlayNode();

    protected static ConfigurationService createCfgServiceMock(final String groupName, final String groupPassword,
            final String host, final String port, final String topologyName, final String containerName)
            throws Exception {
        return createCfgServiceMock(groupName, groupPassword, host, port, topologyName, containerName,
                Collections.<ContainerConfiguration> emptyList());
    }

    /**
     * Create and initialize the mock "Configuration Service"
     */
    protected static ConfigurationService createCfgServiceMock(final String groupName, final String groupPassword,
            final String host, final String port, final String topologyName, final String containerName,
            final Collection<ContainerConfiguration> otherContainers)
            throws Exception {

        final DomainConfiguration domainConfiguration = AbstractRegistryOverlayImplTest
                .createDomainConfiguration(topologyName, groupName, groupPassword, host, port);

        final ContainerConfiguration containerConfiguration = AbstractRegistryOverlayImplTest
                .createContainerConfiguration(containerName);

        return AbstractRegistryOverlayImplTest.createCfgServiceMock(domainConfiguration, containerConfiguration,
                otherContainers);
    }

    protected static ConfigurationService createCfgServiceMock(final DomainConfiguration domainConfiguration,
            final ContainerConfiguration containerConfiguration) throws Exception {
        return createCfgServiceMock(domainConfiguration, containerConfiguration,
                Collections.<ContainerConfiguration> emptyList());
    }

    /**
     * Create and initialize the mock "Configuration Service" from a {@link DomainConfiguration}, a
     * {@link DomainConfiguration} and a {@link ContainerConfiguration}
     */
    protected static ConfigurationService createCfgServiceMock(final DomainConfiguration domainConfiguration,
            final ContainerConfiguration containerConfiguration,
            final Collection<ContainerConfiguration> otherContainers) throws Exception {

        final Collection<ContainerConfiguration> all = new ArrayList<>(otherContainers);
        if (containerConfiguration != null) {
            all.add(containerConfiguration);
        }
        final Topology topology = ConfigurationServiceUtils.toTopology(domainConfiguration, all);

        final ConfigurationService cfgServiceMock = EasyMock.createMock(ConfigurationService.class);
        EasyMock.makeThreadSafe(cfgServiceMock, true);
        EasyMock.expect(cfgServiceMock.getDomainConfiguration()).andReturn(domainConfiguration).anyTimes();
        EasyMock.expect(cfgServiceMock.getContainerConfiguration()).andReturn(containerConfiguration).anyTimes();
        EasyMock.expect(cfgServiceMock.isSecurityTopologyPassphraseOk(VALID_TOPOLOGY_PASSPHRASE)).andReturn(Boolean.TRUE)
                .anyTimes();
        EasyMock.expect(cfgServiceMock.isSecurityTopologyPassphraseOk(INVALID_TOPOLOGY_PASSPHRASE))
                .andReturn(Boolean.FALSE).anyTimes();


        EasyMock.expect(cfgServiceMock.getLocalTopology()).andReturn(topology).anyTimes();
        cfgServiceMock.updateTopology(EasyMock.<Topology> anyObject(), EasyMock.eq(false));
        EasyMock.expectLastCall().anyTimes();
        cfgServiceMock.updateTopology(EasyMock.<Topology> anyObject(), EasyMock.eq(true));
        EasyMock.expectLastCall().times(0, 1);

        EasyMock.replay(cfgServiceMock);

        return cfgServiceMock;
    }

    protected static DomainConfiguration createDomainConfiguration(final String domainName, final String groupName,
            final String groupPassword, final String host, final String port) {

        final DomainConfiguration domainConfiguration = new DomainConfiguration();
        domainConfiguration.setName(domainName);
        domainConfiguration.setDescription("domain description");
        domainConfiguration.setMode(DomainMode.DYNAMIC);

        final List<Object> extraParameters = new ArrayList<Object>();
        if (groupName != null || groupPassword != null || host != null || port != null) {
            // Build the extra parameter
            final Document extraParameter = DocumentBuilders.newDocument();
            final Element rootElt = extraParameter.createElementNS(ConfigurationBuilder.XML.CONFIGURATION_NAMESPACE,
                    ConfigurationBuilder.XML.CONFIGURATION_ROOT_ELEMENT_NAME);
            final Element topologyNameElt = extraParameter.createElementNS(
                    ConfigurationBuilder.XML.CONFIGURATION_NAMESPACE,
                    ConfigurationBuilder.XML.CONFIGURATION_GROUPNAME_ELEMENT_NAME);
            topologyNameElt.setTextContent(groupName);
            rootElt.appendChild(topologyNameElt);
            final Element topologyPwdElt = extraParameter.createElementNS(
                    ConfigurationBuilder.XML.CONFIGURATION_NAMESPACE,
                    ConfigurationBuilder.XML.CONFIGURATION_GROUPPWD_ELEMENT_NAME);
            topologyPwdElt.setTextContent(groupPassword);
            rootElt.appendChild(topologyPwdElt);
            final Element overlayMembers = extraParameter.createElementNS(
                    ConfigurationBuilder.XML.CONFIGURATION_NAMESPACE,
                    ConfigurationBuilder.XML.CONFIGURATION_OVERLAYMEMBERS_ELEMENT_NAME);
            rootElt.appendChild(overlayMembers);
            final Element overlayMember = extraParameter.createElementNS(
                    ConfigurationBuilder.XML.CONFIGURATION_NAMESPACE,
                    ConfigurationBuilder.XML.CONFIGURATION_OVERLAYMEMBER_ELEMENT_NAME);
            overlayMember.setTextContent(host);
            if (port != null) {
                overlayMember.setAttribute(ConfigurationBuilder.XML.CONFIGURATION_OVERLAYMEMBERPORT_ATTR_NAME, port);
            }
            overlayMembers.appendChild(overlayMember);

            extraParameters.add(rootElt);
        }
        domainConfiguration.setExtraParameters(extraParameters);

        return domainConfiguration;
    }

    protected static ContainerConfiguration createContainerConfiguration(final String containerName) {

        final ContainerConfiguration containerConfiguration = new ContainerConfiguration();
        containerConfiguration.setName(containerName);
        containerConfiguration.setTopologyLocksMaxWaitTime(3);

        return containerConfiguration;
    }

    protected static Document readEndpointDescription(final String wsdl) throws WSDL4ComplexWsdlException,
            URISyntaxException, IOException {

        final URL wsdlUrl = Thread.currentThread().getContextClassLoader().getResource(wsdl);
        assertNotNull("WSDL not found", wsdlUrl);
        final WSDL4ComplexWsdlReader wsdl4ComplexWsdlReader = WSDL4ComplexWsdlReaders.takeWSDL4ComplexWsdlReader();
        try {
            final Description wsdlDescription = wsdl4ComplexWsdlReader.read(wsdlUrl);
            wsdlDescription.addImportedDocumentsInWsdl();
            final WSDL4ComplexWsdlWriter wsdl4ComplexWsdlWriter = WSDL4ComplexWsdlWriters.takeWSDL4ComplexWsdlWriter();
            try {
                return wsdl4ComplexWsdlWriter.getDocument(wsdlDescription);

            } finally {
                WSDL4ComplexWsdlWriters.releaseWSDL4ComplexWsdlWriter(wsdl4ComplexWsdlWriter);
            }

        } finally {
            WSDL4ComplexWsdlReaders.releaseWSDL4ComplexWsdlReader(wsdl4ComplexWsdlReader);
        }
    }

    protected static void assertEndpointDescriptionParsable(final RegistryOverlayImpl registry,
            final PetalsServiceEndpoint endpoint)
                    throws WSDL4ComplexWsdlException, URISyntaxException, SharedAreaException {
        if (endpoint.getType().equals(EndpointType.EXTERNAL)) {
            // external endpoints are not concerned by it
            return;
        }
        final Optional<Document> retrievedDocument = registry.getDescription(endpoint);
        assertNotNull("Endpoint is missing", retrievedDocument);
        assertTrue("WSDL description is missing", retrievedDocument.isPresent());
        final WSDL4ComplexWsdlReader anotherComplexWsdlReader = WSDL4ComplexWsdlReaders.takeWSDL4ComplexWsdlReader();
        try {
            final Description wsdlDescription = anotherComplexWsdlReader.read(retrievedDocument.get());
            assertNotNull("EasyWSDL description is null", wsdlDescription);
        } finally {
            WSDL4ComplexWsdlReaders.releaseWSDL4ComplexWsdlReader(anotherComplexWsdlReader);
        }
    }

}
