/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import java.util.Collection;

import javax.xml.namespace.QName;

import org.junit.Test;
import org.ow2.petals.jbi.servicedesc.endpoint.PetalsServiceEndpoint;
import org.ow2.petals.jbi.servicedesc.endpoint.PetalsServiceEndpoint.EndpointType;
import org.ow2.petals.microkernel.api.communication.sharedarea.SharedAreaService;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;
import com.ebmwebsourcing.easycommons.uuid.QualifiedUUIDGenerator;

/**
 * Unit tests of {@link RegistryOverlayImpl#deregisterEndpoint(QName, String, String, EndpointType)}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayImpl_DeactivateEndpoint_Test extends AbstractRegistryOverlayImplTest {

    /**
     * A namespace name
     */
    private final static String A_NAMESPACE = "http://petals.ow2.org/unit-tests";

    /**
     * A service name
     */
    private final static QName SERVICE_NAME_1 = new QName(A_NAMESPACE, "service-1");

    /**
     * A interface name
     */
    private final static QName INTERFACE_NAME_1 = new QName(A_NAMESPACE, "interface-1");

    /**
     * A endpoint name
     */
    private final static String ENDPOINT_NAME_1;

    /**
     * A endpoint name
     */
    private final static String ENDPOINT_NAME_2;

    /**
     * A endpoint name
     */
    private final static String ENDPOINT_NAME_3;

    /**
     * A container name
     */
    private final static String CONTAINER_NAME_1 = "container-1";

    /**
     * A container name
     */
    private final static String CONTAINER_NAME_2 = "container-2";

    static {
        final QualifiedUUIDGenerator uuidGenerator = new QualifiedUUIDGenerator("petals");
        ENDPOINT_NAME_1 = uuidGenerator.getNewID();
        ENDPOINT_NAME_2 = uuidGenerator.getNewID();
        ENDPOINT_NAME_3 = uuidGenerator.getNewID();
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#deregisterEndpoint(QName, String, String, EndpointType)}:
     * </p>
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>deregistration of an endpoint previously correctly registered.</li>
     * </ol>
     * <p>
     * Expected result:
     * </p>
     * <ol>
     * <li>no error occurs, the endpoint is correctly deactivated</li>
     * <li>no more endpoint is retrieved using {@link SharedAreaService#getEndpoint(QName, String, EndpointType)}</li>
     * <li>no more endpoint is retrieved using {@link SharedAreaService#getEndpointsForService(QName, EndpointType)}</li>
     * <li>no more endpoint is retrieved using {@link SharedAreaService#getEndpointsForInterface(QName, EndpointType)}</li>
     * </ol>
     */
    @Test
    public void deregisterOneValidEndpoint() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService",
                RegistryOverlayImpl_DeactivateEndpoint_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass().getSimpleName()
                                + "_deregisterOneValidEndpoint", CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();

            // Register the endpoint
            registryOverlayImpl.registerInternalEndpoint(ENDPOINT_NAME_1, SERVICE_NAME_1,
                    new QName[] { INTERFACE_NAME_1 }, "", null);

            // Retrieve the endpoint
            final PetalsServiceEndpoint retrievedEndpoint = registryOverlayImpl.getEndpoint(SERVICE_NAME_1,
                    ENDPOINT_NAME_1, EndpointType.INTERNAL);
            assertNotNull(retrievedEndpoint);
            assertEquals(retrievedEndpoint.getServiceName(), SERVICE_NAME_1);
            assertEquals(retrievedEndpoint.getEndpointName(), ENDPOINT_NAME_1);

            // Check that only one endpoint is registered as implementing the service
            final Collection<PetalsServiceEndpoint> svcEpForSvc = registryOverlayImpl
                    .getEndpointsForService(SERVICE_NAME_1, null);
            assertNotNull("Null list returned", svcEpForSvc);
            assertEquals("No endpoint retrieved for service", svcEpForSvc.size(), 1);
            assertEquals(svcEpForSvc.iterator().next().getEndpointName(), ENDPOINT_NAME_1);

            // Check that only one endpoint is registered as implementing the interface
            final Collection<PetalsServiceEndpoint> svcEpForItf = registryOverlayImpl
                    .getEndpointsForInterface(INTERFACE_NAME_1, null);
            assertNotNull("Null list returned", svcEpForItf);
            assertEquals("No endpoint retrieved for interface", svcEpForItf.size(), 1);
            assertEquals(svcEpForItf.iterator().next().getEndpointName(), ENDPOINT_NAME_1);

            // Deregister the endpoint
            assertNotNull(
                    registryOverlayImpl.deregisterEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1, "", EndpointType.INTERNAL));

            // Retrieve the endpoint
            assertNull("Endpoint retrieved",
                    registryOverlayImpl.getEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1, EndpointType.INTERNAL));

            // Check that no more endpoint is registered as implementing the service
            final Collection<PetalsServiceEndpoint> noMoreSvcEpForSvc = registryOverlayImpl
                    .getEndpointsForService(SERVICE_NAME_1, null);
            assertNotNull("Null list returned", noMoreSvcEpForSvc);
            assertEquals("Endpoint retrieved for service", noMoreSvcEpForSvc.size(), 0);

            // Check that no more endpoint is registered as implementing the interface
            final Collection<PetalsServiceEndpoint> noMoreSvcEpForItf = registryOverlayImpl
                    .getEndpointsForInterface(INTERFACE_NAME_1, null);
            assertNotNull("Null list returned", noMoreSvcEpForItf);
            assertEquals("Endpoint retrieved for interface", noMoreSvcEpForItf.size(), 0);

        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#deregisterEndpoint(QName, String, String, EndpointType)}:
     * </p>
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>deactivation of an endpoint not previously activated.</li>
     * </ol>
     * <p>
     * Expected result:
     * </p>
     * <ol>
     * <li>an error occurs, the endpoint does not exist</li>
     * </ol>
     */
    @Test
    public void deregisterAnNotExistinglidEndpoint() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService",
                RegistryOverlayImpl_DeactivateEndpoint_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass().getSimpleName()
                                + "_deregisterOneValidEndpoint", CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();

            // Deregister the endpoint
            assertNull(
                    registryOverlayImpl.deregisterEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1, "", EndpointType.INTERNAL));

        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#deregisterEndpoint(QName, String, String, EndpointType)}:
     * </p>
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>deregistration of an endpoint previously correctly registered.</li>
     * </ol>
     * <p>
     * Expected result:
     * </p>
     * <ol>
     * <li>no error occurs, the endpoint is correctly deactivated</li>
     * <li>no more endpoint is retrieved using {@link SharedAreaService#getEndpoint(QName, String, EndpointType)}</li>
     * <li>no more endpoint is retrieved using {@link SharedAreaService#getEndpointsForService(QName, EndpointType)}
     * </li>
     * <li>no more endpoint is retrieved using {@link SharedAreaService#getEndpointsForInterface(QName, EndpointType)}
     * </li>
     * </ol>
     */
    @Test
    public void deregisterEndpointNotOwned() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService",
                RegistryOverlayImpl_DeactivateEndpoint_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()),
                        this.getClass().getSimpleName() + "_deregisterOneValidEndpoint", CONTAINER_NAME_1));

        final RegistryOverlayImpl registryOverlayImpl2 = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl2, "configurationService",
                RegistryOverlayImpl_DeactivateEndpoint_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()),
                        this.getClass().getSimpleName() + "_deregisterOneValidEndpoint", CONTAINER_NAME_2));
        try {
            registryOverlayImpl.start();
            registryOverlayImpl2.start();

            // same name but two different containers!
            final String componentOn1 = "component1";
            final String component1On2 = "component1";
            final String component2On2 = "component2";

            // Register the endpoints
            registryOverlayImpl.registerInternalEndpoint(ENDPOINT_NAME_1, SERVICE_NAME_1,
                    new QName[] { INTERFACE_NAME_1 }, componentOn1, null);
            registryOverlayImpl2.registerInternalEndpoint(ENDPOINT_NAME_2, SERVICE_NAME_1,
                    new QName[] { INTERFACE_NAME_1 }, component1On2, null);
            registryOverlayImpl2.registerInternalEndpoint(ENDPOINT_NAME_3, SERVICE_NAME_1,
                    new QName[] { INTERFACE_NAME_1 }, component2On2, null);

            final PetalsServiceEndpoint retrievedEndpoint1 = registryOverlayImpl.getEndpoint(SERVICE_NAME_1,
                    ENDPOINT_NAME_1, EndpointType.INTERNAL);
            assertNotNull(retrievedEndpoint1);
            assertEquals(CONTAINER_NAME_1, retrievedEndpoint1.getLocation().getContainerName());
            assertEquals(componentOn1, retrievedEndpoint1.getLocation().getComponentName());

            final PetalsServiceEndpoint retrievedEndpoint2 = registryOverlayImpl.getEndpoint(SERVICE_NAME_1,
                    ENDPOINT_NAME_2, EndpointType.INTERNAL);
            assertNotNull(retrievedEndpoint2);
            assertEquals(CONTAINER_NAME_2, retrievedEndpoint2.getLocation().getContainerName());
            assertEquals(component1On2, retrievedEndpoint2.getLocation().getComponentName());

            final PetalsServiceEndpoint retrievedEndpoint3 = registryOverlayImpl.getEndpoint(SERVICE_NAME_1,
                    ENDPOINT_NAME_3, EndpointType.INTERNAL);
            assertNotNull(retrievedEndpoint3);
            assertEquals(CONTAINER_NAME_2, retrievedEndpoint3.getLocation().getContainerName());
            assertEquals(component2On2, retrievedEndpoint3.getLocation().getComponentName());

            // Can't deregister endpoint on other container as wrong container/component
            assertNull(registryOverlayImpl.deregisterEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_2, componentOn1,
                    EndpointType.INTERNAL));

            assertNotNull(registryOverlayImpl.getEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_2, EndpointType.INTERNAL));

            // Can't deregister endpoint on other container as wrong container/component
            assertNull(registryOverlayImpl2.deregisterEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1, component1On2,
                    EndpointType.INTERNAL));

            assertNotNull(registryOverlayImpl.getEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1, EndpointType.INTERNAL));

            // Can't deregister endpoint on other component as wrong component
            assertNull(registryOverlayImpl2.deregisterEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_2, component2On2,
                    EndpointType.INTERNAL));

            assertNotNull(registryOverlayImpl.getEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_2, EndpointType.INTERNAL));

            assertNotNull(registryOverlayImpl.deregisterEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1, componentOn1,
                    EndpointType.INTERNAL));
            assertNotNull(registryOverlayImpl2.deregisterEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_2, component1On2,
                    EndpointType.INTERNAL));
            assertNotNull(registryOverlayImpl2.deregisterEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_3, component2On2,
                    EndpointType.INTERNAL));

        } finally {
            registryOverlayImpl.stop();
        }
    }

}
