/**
 * Copyright (c) 2014-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import java.util.Properties;

import org.junit.Test;
import org.ow2.petals.microkernel.registry.overlay.api.RegistryOverlayAPI;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Unit tests of {@link RegistryOverlayImpl#getConfiguration(String)}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayImpl_GetConfiguration_Test extends AbstractRegistryOverlayImplTest {

    /**
     * Name of the topoplogy used by unit tests
     */
    private final static String TOPOLOGY_NAME = "My Topology Name";

    /**
     * A container name
     */
    private final static String CONTAINER_NAME_1 = "container-1";

    private final static String CONF_PREFIX = RegistryOverlayImpl.class.getName() + ".";

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getConfiguration(String)} when the following configuration is
     * previously loaded successfully:
     * </p>
     * <ul>
     * <li>only one member</li>
     * <li>topology pass-phrase is valid</li>
     * </ul>
     * <p>
     * Expected result:
     * </p>
     * <ul>
     * <li>no error occurs</li>
     * <li>the group password is not returned because of invalid topology pass-phrase</li>
     * </ul>
     */
    @Test
    public void testValidPassphrase() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService",
                RegistryOverlayImpl_GetConfiguration_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), TOPOLOGY_NAME, CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();

            final Properties config = registryOverlayImpl.getConfiguration(VALID_TOPOLOGY_PASSPHRASE);

            assertEquals("Unexpected group name", PETALS_REGISTRY_NODE.getGroupName(),
                    config.getProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_NAME));
            assertEquals("Group password returned", PETALS_REGISTRY_NODE.getPassword(),
                    config.getProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_PASSWORD));

            int address = 0;
            for (final String propertyName : config.stringPropertyNames()) {
                if (propertyName.startsWith(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.ADDRESS + ".")) {
                    address++;
                }
            }
            assertEquals("Unexpected address number", 1, address);
            assertEquals("Unexpected address",
                    PETALS_REGISTRY_NODE.getHost() + ":" + String.valueOf(PETALS_REGISTRY_NODE.getPort()),
                    config.getProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.ADDRESS + ".0"));
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getConfiguration(String)} when the following configuration is
     * previously loaded successfully:
     * </p>
     * <ul>
     * <li>only one member</li>
     * <li>topology pass-phrase is invalid</li>
     * </ul>
     * <p>
     * Expected result:
     * </p>
     * <ul>
     * <li>no error occurs</li>
     * <li>the group password is not returned because of invalid topology pass-phrase</li>
     * </ul>
     */
    @Test
    public void testInvalidPassphrase() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService",
                RegistryOverlayImpl_GetConfiguration_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), TOPOLOGY_NAME, CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();

            final Properties config = registryOverlayImpl.getConfiguration(INVALID_TOPOLOGY_PASSPHRASE);

            assertEquals("Unexpected group name", PETALS_REGISTRY_NODE.getGroupName(),
                    config.getProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_NAME));
            assertNull("Group password returned",
                    config.getProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_PASSWORD));

            int address = 0;
            for (final String propertyName : config.stringPropertyNames()) {
                if (propertyName.startsWith(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.ADDRESS + ".")) {
                    address++;
                }
            }
            assertEquals("Unexpected address number", 1, address);
            assertEquals("Unexpected address",
                    PETALS_REGISTRY_NODE.getHost() + ":" + String.valueOf(PETALS_REGISTRY_NODE.getPort()),
                    config.getProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.ADDRESS + ".0"));
        } finally {
            registryOverlayImpl.stop();
        }
    }

}
