/**
 * Copyright (c) 2014-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.List;
import java.util.Properties;

import org.junit.Test;
import org.ow2.petals.microkernel.api.communication.sharedarea.exception.InvalidPropertyException;
import org.ow2.petals.microkernel.api.communication.sharedarea.exception.MissingPropertyException;
import org.ow2.petals.microkernel.api.implementation.exception.MutableImplException;
import org.ow2.petals.microkernel.registry.overlay.api.RegistryOverlayAPI;
import org.ow2.petals.registry_overlay.client.configuration.ConfigurationBuilder;
import org.ow2.petals.registry_overlay.client.configuration.ConfigurationException;
import org.ow2.petals.registry_overlay.client.configuration.generated.Configuration;
import org.ow2.petals.registry_overlay.client.configuration.generated.OverlayMember;
import org.ow2.petals.registry_overlay.client.configuration.generated.OverlayMembers;
import org.w3c.dom.Document;

/**
 * Unit tests of {@link RegistryOverlayImpl#getTopologyPartConfiguration(Properties)}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayImpl_GetTopologyPartConfiguration_Test {

    /**
     * Name of the registry group (ie. login of the Hazelcast cluster)
     */
    private final static String REGISTRY_GROUP_NAME = "dev";

    /**
     * Password of the registry group (ie. password of the Hazelcast cluster)
     */
    private final static String REGISTRY_GROUP_PASSWORD = "s3cr3t";

    /**
     * Host name of a hazelcast cluster member
     */
    private final static String HZL_CLUSTER_HOST = "localhost";

    /**
     * Host name of another hazelcast cluster member
     */
    private final static String HZL_CLUSTER_HOST_1 = "localhost-2";

    /**
     * Port of the second hazelcast cluster member
     */
    private final static String HZL_CLUSTER_PORT_1 = "12345";

    private final static String CONF_PREFIX = RegistryOverlayImpl.class.getName() + ".";

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getTopologyPartConfiguration(Properties)} where the properties are
     * <code>null</code>
     * </p>
     * <p>
     * Expected result:
     * </p>
     * <ul>
     * <li>no error occurs</li>
     * <li><code>null</code> is returned</li>
     * </ul>
     */
    @Test
    public void testPropertiesNull() throws MutableImplException {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();

        assertNull("Not null result returned", registryOverlayImpl.getTopologyPartConfiguration(null));
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getTopologyPartConfiguration(Properties)} where the group name is
     * missing in the properties
     * </p>
     * <p>
     * Expected result: an error occurs about the missing group name
     * </p>
     */
    @Test(expected = MissingPropertyException.class)
    public void testMissingGroupName() throws MutableImplException {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        final Properties properties = new Properties();

        try {
            registryOverlayImpl.getTopologyPartConfiguration(properties);
        } catch (final MissingPropertyException e) {
            assertEquals("Unexpected missing property", CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_NAME,
                    e.getPropertyName());
            throw e;
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getTopologyPartConfiguration(Properties)} where the group password is
     * missing in the properties
     * </p>
     * <p>
     * Expected result: an error occurs about the missing group password
     * </p>
     */
    @Test(expected = MissingPropertyException.class)
    public void testMissingGroupPassword() throws MutableImplException {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        final Properties properties = new Properties();
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_NAME, REGISTRY_GROUP_NAME);

        try {
            registryOverlayImpl.getTopologyPartConfiguration(properties);
        } catch (final MissingPropertyException e) {
            assertEquals("Unexpected missing property", CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_PASSWORD,
                    e.getPropertyName());
            throw e;
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getTopologyPartConfiguration(Properties)} where no overlay member is
     * set in the properties
     * </p>
     * <p>
     * Expected result: an error occurs about the missing overlay member
     * </p>
     */
    @Test(expected = MissingPropertyException.class)
    public void testMissingOverlayMember() throws MutableImplException {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        final Properties properties = new Properties();
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_NAME, REGISTRY_GROUP_NAME);
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_PASSWORD, REGISTRY_GROUP_PASSWORD);

        try {
            registryOverlayImpl.getTopologyPartConfiguration(properties);
        } catch (final MissingPropertyException e) {
            assertTrue("Unexpected missing property",
                    e.getPropertyName().startsWith(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.ADDRESS));
            throw e;
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getTopologyPartConfiguration(Properties)} where no host (empty value)
     * is set in the overlay member in the properties
     * </p>
     * <p>
     * Expected result: an error occurs about the invalid value of the property associated to the overlay member
     * </p>
     */
    @Test(expected = InvalidPropertyException.class)
    public void testEmptyOverlayMember() throws MutableImplException {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        final Properties properties = new Properties();
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_NAME, REGISTRY_GROUP_NAME);
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_PASSWORD, REGISTRY_GROUP_PASSWORD);
        final String addressPropertyName = CONF_PREFIX + RegistryOverlayAPI.RegistryItem.ADDRESS + ".0";
        final String invalidPropertyValue = "";
        properties.setProperty(addressPropertyName, invalidPropertyValue);

        try {
            registryOverlayImpl.getTopologyPartConfiguration(properties);
        } catch (final InvalidPropertyException e) {
            assertEquals("Unexpected invalid property", addressPropertyName, e.getPropertyName());
            assertEquals("Unexpected invalid property value", invalidPropertyValue, e.getInvalidValue());
            throw e;
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getTopologyPartConfiguration(Properties)} where no host (only port)
     * is set in the overlay member in the properties
     * </p>
     * <p>
     * Expected result: an error occurs about the invalid value of the property associated to the overlay member
     * </p>
     */
    @Test(expected = InvalidPropertyException.class)
    public void testMissingOverlayMemberHostName() throws MutableImplException {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        final Properties properties = new Properties();
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_NAME, REGISTRY_GROUP_NAME);
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_PASSWORD, REGISTRY_GROUP_PASSWORD);
        final String addressPropertyName = CONF_PREFIX + RegistryOverlayAPI.RegistryItem.ADDRESS + ".0";
        final String invalidPropertyValue = ":12345";
        properties.setProperty(addressPropertyName, invalidPropertyValue);

        try {
            registryOverlayImpl.getTopologyPartConfiguration(properties);
        } catch (final InvalidPropertyException e) {
            assertEquals("Unexpected invalid property", addressPropertyName, e.getPropertyName());
            assertEquals("Unexpected invalid property value", invalidPropertyValue, e.getInvalidValue());
            throw e;
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getTopologyPartConfiguration(Properties)} where an invalid port value
     * is set in the overlay member in the properties
     * </p>
     * <p>
     * Expected result: an error occurs about the invalid value of the property associated to the overlay member
     * </p>
     */
    @Test(expected = InvalidPropertyException.class)
    public void testInvaliOverlayMemberPort() throws MutableImplException {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        final Properties properties = new Properties();
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_NAME, REGISTRY_GROUP_NAME);
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_PASSWORD, REGISTRY_GROUP_PASSWORD);
        final String addressPropertyName = CONF_PREFIX + RegistryOverlayAPI.RegistryItem.ADDRESS + ".0";
        final String invalidPropertyValue = ":invalid-port";
        properties.setProperty(addressPropertyName, invalidPropertyValue);

        try {
            registryOverlayImpl.getTopologyPartConfiguration(properties);
        } catch (final InvalidPropertyException e) {
            assertEquals("Unexpected invalid property", addressPropertyName, e.getPropertyName());
            assertEquals("Unexpected invalid property value", invalidPropertyValue, e.getInvalidValue());
            throw e;
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getTopologyPartConfiguration(Properties)} where an invalid value (too
     * much fields) is set in the overlay member in the properties
     * </p>
     * <p>
     * Expected result: an error occurs about the invalid value of the property associated to the overlay member
     * </p>
     */
    @Test(expected = InvalidPropertyException.class)
    public void testInvaliOverlayMemberTooFields() throws MutableImplException {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        final Properties properties = new Properties();
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_NAME, REGISTRY_GROUP_NAME);
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_PASSWORD, REGISTRY_GROUP_PASSWORD);
        final String addressPropertyName = CONF_PREFIX + RegistryOverlayAPI.RegistryItem.ADDRESS + ".0";
        final String invalidPropertyValue = HZL_CLUSTER_HOST_1 + ":" + HZL_CLUSTER_PORT_1 + ":another-field";
        properties.setProperty(addressPropertyName, invalidPropertyValue);

        try {
            registryOverlayImpl.getTopologyPartConfiguration(properties);
        } catch (final InvalidPropertyException e) {
            assertEquals("Unexpected invalid property", addressPropertyName, e.getPropertyName());
            assertEquals("Unexpected invalid property value", invalidPropertyValue, e.getInvalidValue());
            throw e;
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getTopologyPartConfiguration(Properties)} when all the required
     * properties are provided
     * </p>
     * <p>
     * Expected result:
     * </p>
     * <ul>
     * <li>no error occurs</li>
     * <li>the XML is parsable and it is correct</li>
     * </ul>
     */
    @Test
    public void testRequiredProperties() throws MutableImplException, ConfigurationException {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        final Properties properties = new Properties();
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_NAME, REGISTRY_GROUP_NAME);
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_PASSWORD, REGISTRY_GROUP_PASSWORD);
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.ADDRESS + ".0", HZL_CLUSTER_HOST);

        final Document document = registryOverlayImpl.getTopologyPartConfiguration(properties);

        final Configuration generatedConfiguration = ConfigurationBuilder.createConfiguration(document);

        assertNotNull("Null configuration returned", generatedConfiguration);
        assertEquals("Unexpected group name", REGISTRY_GROUP_NAME, generatedConfiguration.getGroupName());
        assertEquals("Unexpected group pwd", REGISTRY_GROUP_PASSWORD, generatedConfiguration.getGroupPassword());
        final OverlayMembers overlayMembers = generatedConfiguration.getOverlayMembers();
        assertNotNull("Overlay members is null", overlayMembers);
        final List<OverlayMember> overlayMemberList = overlayMembers.getOverlayMember();
        assertNotNull("Overlay member list is null", overlayMemberList);
        assertEquals("Unexpected number of overlay member", 1, overlayMemberList.size());
        assertEquals("Unexpected overlay member hostname", HZL_CLUSTER_HOST, overlayMemberList.get(0).getValue());
        assertNull("An overlay member port is set", overlayMemberList.get(0).getPort());
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getTopologyPartConfiguration(Properties)} when all the required
     * properties are provided, plus:
     * </p>
     * <ul>
     * <li>a second overlay member is given</li>
     * <li>the second overlay member has a given port,</li>
     * </ul>
     * <p>
     * Expected result:
     * </p>
     * <ul>
     * <li>no error occurs</li>
     * <li>the XML is parsable and it is correct</li>
     * </ul>
     */
    @Test
    public void testRequiredPropertiesExtended() throws MutableImplException, ConfigurationException {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        final Properties properties = new Properties();
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_NAME, REGISTRY_GROUP_NAME);
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.GROUP_PASSWORD, REGISTRY_GROUP_PASSWORD);
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.ADDRESS + ".0", HZL_CLUSTER_HOST);
        properties.setProperty(CONF_PREFIX + RegistryOverlayAPI.RegistryItem.ADDRESS + ".1", HZL_CLUSTER_HOST_1 + ":"
                + HZL_CLUSTER_PORT_1);

        final Document document = registryOverlayImpl.getTopologyPartConfiguration(properties);

        final Configuration generatedConfiguration = ConfigurationBuilder.createConfiguration(document);

        assertNotNull("Null configuration returned", generatedConfiguration);
        assertEquals("Unexpected group name", REGISTRY_GROUP_NAME, generatedConfiguration.getGroupName());
        assertEquals("Unexpected group pwd", REGISTRY_GROUP_PASSWORD, generatedConfiguration.getGroupPassword());
        final OverlayMembers overlayMembers = generatedConfiguration.getOverlayMembers();
        assertNotNull("Overlay members is null", overlayMembers);
        final List<OverlayMember> overlayMemberList = overlayMembers.getOverlayMember();
        assertNotNull("Overlay member list is null", overlayMemberList);
        assertEquals("Unexpected number of overlay member", 2, overlayMemberList.size());
        boolean isMemberOneFound = false;
        boolean isMember2ndFound = false;
        for (final OverlayMember member : overlayMemberList) {
            if (member.getValue().equals(HZL_CLUSTER_HOST)) {
                assertNull("An overlay member port is set", member.getPort());
                isMemberOneFound = true;
            } else if (member.getValue().equals(HZL_CLUSTER_HOST_1)) {
                assertEquals("Unexpected overlay member port", HZL_CLUSTER_PORT_1, member.getPort().toString());
                isMember2ndFound = true;
            }
        }
        assertTrue("Overlay member 1 not found", isMemberOneFound);
        assertTrue("Overlay member 2 not found", isMember2ndFound);

    }

}
