/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.junit.Ignore;
import org.junit.Test;
import org.ow2.petals.microkernel.api.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration.DomainMode;
import org.ow2.petals.microkernel.registry.overlay.exception.AuthenticationException;
import org.ow2.petals.microkernel.registry.overlay.exception.InvalidConfigurationException;
import org.ow2.petals.registry_overlay.api.Constants;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;
import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.hazelcast.client.impl.HazelcastClientProxy;
import com.hazelcast.config.Config;
import com.hazelcast.config.NearCacheConfig;
import com.hazelcast.config.NetworkConfig;
import com.hazelcast.core.Hazelcast;

/**
 * Unit tests of {@link RegistryOverlayImpl#start()}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayImpl_Start_Test extends AbstractRegistryOverlayImplTest {

    /**
     * Name of the topology used by unit tests
     */
    private final static String TOPOLOGY_NAME = "My Topology Name";

    /**
     * A container name
     */
    private final static String CONTAINER_NAME_1 = "container-1";

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a valid configuration is provided.
     * </p>
     * <p>
     * Expected result: no error occurs
     * </p>
     */
    @Test
    public void testConfiguration_Is_Valid() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(), PETALS_REGISTRY_NODE.getPassword(),
                        PETALS_REGISTRY_NODE.getHost(), String.valueOf(PETALS_REGISTRY_NODE.getPort()), TOPOLOGY_NAME,
                        CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a invalid configuration is provided: no group name is
     * provided
     * </p>
     * <p>
     * Expected result: an error occurs
     * </p>
     */
    @Test(expected = InvalidConfigurationException.class)
    public void testConfiguration_Is_Invalid_00() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock(null, PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), TOPOLOGY_NAME, CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a invalid configuration is provided: the group name is
     * empty
     * </p>
     * <p>
     * Expected result: an error occurs
     * </p>
     */
    @Test(expected = InvalidConfigurationException.class)
    public void testConfiguration_Is_Invalid_01() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock("", PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), TOPOLOGY_NAME, CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a invalid configuration is provided: no group password
     * is provided
     * </p>
     * <p>
     * Expected result: an error occurs
     * </p>
     */
    @Test(expected = InvalidConfigurationException.class)
    public void testConfiguration_Is_Invalid_02() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(), null, PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), TOPOLOGY_NAME, CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a invalid configuration is provided: the group password
     * is empty
     * </p>
     * <p>
     * Expected result: an error occurs
     * </p>
     */
    @Test(expected = InvalidConfigurationException.class)
    public void testConfiguration_Is_Invalid_03() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(), "", PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), TOPOLOGY_NAME, CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a invalid configuration is provided: no overlay member
     * hostname is provided
     * </p>
     * <p>
     * Expected result: an error occurs
     * </p>
     */
    @Test(expected = InvalidConfigurationException.class)
    public void testConfiguration_Is_Invalid_04() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(), PETALS_REGISTRY_NODE.getPassword(), null,
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), TOPOLOGY_NAME, CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a invalid configuration is provided: the overlay member
     * hostname is empty
     * </p>
     * <p>
     * Expected result: an error occurs
     * </p>
     */
    @Test(expected = InvalidConfigurationException.class)
    public void testConfiguration_Is_Invalid_05() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(), PETALS_REGISTRY_NODE.getPassword(), "",
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), TOPOLOGY_NAME, CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a invalid configuration is provided: no overlay member
     * port is provided
     * </p>
     * <p>
     * Expected result: no error occurs, the default port is used
     * </p>
     */
    @Test
    public void testConfiguration_Is_Invalid_06() throws Exception {

        // Create the cluster member listening on default port. It will be shutdown by
        // AbstractRegistryOverlayImplTest.stopHazelcastCluster
        final Config hzlConfig = new Config();
        final NetworkConfig networkConfig = new NetworkConfig();
        hzlConfig.setNetworkConfig(networkConfig);
        networkConfig.setPort(Constants.DEFAULT_PORT);
        Hazelcast.newHazelcastInstance(hzlConfig);

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(), PETALS_REGISTRY_NODE.getPassword(),
                        PETALS_REGISTRY_NODE.getHost(), null, TOPOLOGY_NAME, CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();
            final HazelcastClientProxy hzlClient = (HazelcastClientProxy) ReflectionHelper.getPrivateFieldValue(
                    RegistryOverlayImpl.class, registryOverlayImpl, "hzlClient");
            assertEquals("More than one address", 1, hzlClient.getClientConfig().getNetworkConfig().getAddresses()
                    .size());
            assertTrue("Default port no used", hzlClient.getClientConfig().getNetworkConfig().getAddresses().get(0)
                    .endsWith(":" + Constants.DEFAULT_PORT));

        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a invalid configuration is provided: the overlay member
     * port is empty
     * </p>
     * <p>
     * Expected result: an error occurs
     * </p>
     */
    @Test(expected = InvalidConfigurationException.class)
    public void testConfiguration_Is_Invalid_07() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(), PETALS_REGISTRY_NODE.getPassword(),
                        PETALS_REGISTRY_NODE.getHost(), "", TOPOLOGY_NAME, CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a invalid configuration is provided: no extra parameter
     * is defined (ie. {@link DomainConfiguration#getExtraParameters()} returns <code>null</code>)
     * </p>
     * <p>
     * Expected result: an error occurs
     * </p>
     */
    @Test(expected = InvalidConfigurationException.class)
    public void testConfiguration_Is_Invalid_08() throws Exception {


        final DomainConfiguration domainConfiguration = new DomainConfiguration();
        domainConfiguration.setName(TOPOLOGY_NAME);
        // this is needed by the mock creation
        domainConfiguration.setMode(DomainMode.DYNAMIC);
        domainConfiguration.setExtraParameters(null);

        final ContainerConfiguration containerConfiguration = new ContainerConfiguration();
        containerConfiguration.setName(CONTAINER_NAME_1);

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", AbstractRegistryOverlayImplTest
                .createCfgServiceMock(domainConfiguration, containerConfiguration));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a invalid configuration is provided: no extra parameter
     * about registry is defined (ie. {@link DomainConfiguration#getExtraParameters()} returns an empty array)
     * </p>
     * <p>
     * Expected result: an error occurs
     * </p>
     */
    @Test(expected = InvalidConfigurationException.class)
    public void testConfiguration_Is_Invalid_09() throws Exception {

        final DomainConfiguration domainConfiguration = new DomainConfiguration();
        domainConfiguration.setName(TOPOLOGY_NAME);
        // this is needed by the mock creation
        domainConfiguration.setMode(DomainMode.DYNAMIC);
        domainConfiguration.setExtraParameters(new ArrayList<Object>());

        final ContainerConfiguration containerConfiguration = new ContainerConfiguration();
        containerConfiguration.setName(CONTAINER_NAME_1);

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", AbstractRegistryOverlayImplTest
                .createCfgServiceMock(domainConfiguration, containerConfiguration));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a invalid configuration is provided: extra parameter of
     * the domain contains something but nothing about the registry (ie.
     * {@link DomainConfiguration#getExtraParameters()} returns an non-empty array without item about registry)
     * </p>
     * <p>
     * Expected result: an error occurs
     * </p>
     */
    @Test(expected = InvalidConfigurationException.class)
    public void testConfiguration_Is_Invalid_10() throws Exception {


        final List<Object> extraParameters = new ArrayList<Object>();
        final Document extraParameter = DocumentBuilders.newDocument();
        final Element rootElt = extraParameter.createElementNS("http://a.dummy.namespace", "an-unexpected-element");
        extraParameters.add(rootElt);

        final DomainConfiguration domainConfiguration = new DomainConfiguration();
        domainConfiguration.setName(TOPOLOGY_NAME);
        // this is needed by the mock creation
        domainConfiguration.setMode(DomainMode.DYNAMIC);
        domainConfiguration.setExtraParameters(extraParameters);

        final ContainerConfiguration containerConfiguration = new ContainerConfiguration();
        containerConfiguration.setName(CONTAINER_NAME_1);

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", AbstractRegistryOverlayImplTest
                .createCfgServiceMock(domainConfiguration, containerConfiguration));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when an invalid configuration is provided: invalid
     * credentials
     * </p>
     * <p>
     * Expected result: an error occurs
     * </p>
     */
    @Test(expected = AuthenticationException.class)
    @Ignore(value = "Unit test ignored until https://github.com/hazelcast/hazelcast/issues/3769, https://github.com/hazelcast/hazelcast/issues/3772 are not fixed.")
    public void testConfiguration_Invalid_Credentials() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock("bad-name", "no-password", PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), TOPOLOGY_NAME, CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} when a invalid configuration is provided: no domain name
     * configured (ie. {@link DomainConfiguration#getName()} returns null)
     * </p>
     * <p>
     * Expected result: an error occurs
     * </p>
     */
    @Test(expected = InvalidConfigurationException.class)
    public void testConfigurationInvalid_NoDomainName() throws Exception {

        final DomainConfiguration domainConfiguration = new DomainConfiguration();
        // this is needed by the mock creation
        domainConfiguration.setMode(DomainMode.DYNAMIC);

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService",
                AbstractRegistryOverlayImplTest.createCfgServiceMock(domainConfiguration, null));
        try {
            registryOverlayImpl.start();
        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} against the configuration of the max size of the map caches
     * ({@link RegistryOverlayImpl#PROPERTY_NAME_MAP_CACHE_MAX_SIZE}):
     * </p>
     * <ul>
     * <li>max size not set</li>
     * </ul>
     * <p>
     * Expected result: no error occurs, the default value is used
     * </p>
     */
    @Test
    public void testConfiguration_MapCacheNotSet() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(), PETALS_REGISTRY_NODE.getPassword(),
                        PETALS_REGISTRY_NODE.getHost(), String.valueOf(PETALS_REGISTRY_NODE.getPort()), TOPOLOGY_NAME,
                        CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();
            final HazelcastClientProxy hzlClient = (HazelcastClientProxy) ReflectionHelper.getPrivateFieldValue(
                    RegistryOverlayImpl.class, registryOverlayImpl, "hzlClient");
            final Map<String, NearCacheConfig> nearCacheConfigs = hzlClient.getClientConfig().getNearCacheConfigMap();
            assertNotNull("No near cache configuration available", nearCacheConfigs);
            final Iterator<NearCacheConfig> itNearCacheConfigs = nearCacheConfigs.values().iterator();
            while (itNearCacheConfigs.hasNext()) {
                assertEquals(RegistryOverlayImpl.DEFAULT_MAP_CACHE_MAX_SIZE, itNearCacheConfigs.next().getMaxSize());
            }

        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} against the configuration of the max size of the map caches
     * ({@link RegistryOverlayImpl#PROPERTY_NAME_MAP_CACHE_MAX_SIZE}):
     * </p>
     * <ul>
     * <li>max size is set to an invalid value</li>
     * </ul>
     * <p>
     * Expected result: no error occurs, the default value is used
     * </p>
     */
    @Test
    public void testConfiguration_MapCacheSetInvalidValue() throws Exception {

        final DomainConfiguration domainConfiguration = AbstractRegistryOverlayImplTest.createDomainConfiguration(
                TOPOLOGY_NAME, PETALS_REGISTRY_NODE.getGroupName(), PETALS_REGISTRY_NODE.getPassword(),
                PETALS_REGISTRY_NODE.getHost(), String.valueOf(PETALS_REGISTRY_NODE.getPort()));

        final ContainerConfiguration containerConfiguration = new ContainerConfiguration();
        containerConfiguration.setName(CONTAINER_NAME_1);
        final Map<String, String> extraConfiguration = new HashMap<String, String>();
        extraConfiguration.put(RegistryOverlayImpl.PROPERTY_NAME_MAP_CACHE_MAX_SIZE, "invalid-value");
        containerConfiguration.setExtraConfiguration(extraConfiguration);

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock(domainConfiguration, containerConfiguration));
        try {
            registryOverlayImpl.start();
            final HazelcastClientProxy hzlClient = (HazelcastClientProxy) ReflectionHelper.getPrivateFieldValue(
                    RegistryOverlayImpl.class, registryOverlayImpl, "hzlClient");
            final Map<String, NearCacheConfig> nearCacheConfigs = hzlClient.getClientConfig().getNearCacheConfigMap();
            assertNotNull("No near cache configuration available", nearCacheConfigs);
            final Iterator<NearCacheConfig> itNearCacheConfigs = nearCacheConfigs.values().iterator();
            while (itNearCacheConfigs.hasNext()) {
                assertEquals(RegistryOverlayImpl.DEFAULT_MAP_CACHE_MAX_SIZE, itNearCacheConfigs.next().getMaxSize());
            }

        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} against the configuration of the max size of the map caches
     * ({@link RegistryOverlayImpl#PROPERTY_NAME_MAP_CACHE_MAX_SIZE}):
     * </p>
     * <ul>
     * <li>max size is set to a valid value</li>
     * </ul>
     * <p>
     * Expected result: no error occurs, the default value is used
     * </p>
     */
    @Test
    public void testConfiguration_MapCacheSetValidValue() throws Exception {

        final DomainConfiguration domainConfiguration = AbstractRegistryOverlayImplTest.createDomainConfiguration(
                TOPOLOGY_NAME, PETALS_REGISTRY_NODE.getGroupName(), PETALS_REGISTRY_NODE.getPassword(),
                PETALS_REGISTRY_NODE.getHost(), String.valueOf(PETALS_REGISTRY_NODE.getPort()));

        final ContainerConfiguration containerConfiguration = new ContainerConfiguration();
        containerConfiguration.setName(CONTAINER_NAME_1);
        final Map<String, String> extraConfiguration = new HashMap<String, String>();
        final int expectedValue = 1500;
        extraConfiguration.put(RegistryOverlayImpl.PROPERTY_NAME_MAP_CACHE_MAX_SIZE, Integer.toString(expectedValue));
        containerConfiguration.setExtraConfiguration(extraConfiguration);

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock(domainConfiguration, containerConfiguration));
        try {
            registryOverlayImpl.start();
            final HazelcastClientProxy hzlClient = (HazelcastClientProxy) ReflectionHelper.getPrivateFieldValue(
                    RegistryOverlayImpl.class, registryOverlayImpl, "hzlClient");
            final Map<String, NearCacheConfig> nearCacheConfigs = hzlClient.getClientConfig().getNearCacheConfigMap();
            assertNotNull("No near cache configuration available", nearCacheConfigs);
            final Iterator<NearCacheConfig> itNearCacheConfigs = nearCacheConfigs.values().iterator();
            while (itNearCacheConfigs.hasNext()) {
                assertEquals(expectedValue, itNearCacheConfigs.next().getMaxSize());
            }

        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#start()} against the configuration of the max size of the map caches
     * ({@link RegistryOverlayImpl#PROPERTY_NAME_MAP_CACHE_MAX_SIZE}):
     * </p>
     * <ul>
     * <li>max size is set to an empty value</li>
     * </ul>
     * <p>
     * Expected result: no error occurs, the default value is used
     * </p>
     */
    @Test
    public void testConfiguration_MapCacheSetEmptyValue() throws Exception {

        final DomainConfiguration domainConfiguration = AbstractRegistryOverlayImplTest.createDomainConfiguration(
                TOPOLOGY_NAME, PETALS_REGISTRY_NODE.getGroupName(), PETALS_REGISTRY_NODE.getPassword(),
                PETALS_REGISTRY_NODE.getHost(), String.valueOf(PETALS_REGISTRY_NODE.getPort()));

        final ContainerConfiguration containerConfiguration = new ContainerConfiguration();
        containerConfiguration.setName(CONTAINER_NAME_1);
        final Map<String, String> extraConfiguration = new HashMap<String, String>();
        extraConfiguration.put(RegistryOverlayImpl.PROPERTY_NAME_MAP_CACHE_MAX_SIZE, "");
        containerConfiguration.setExtraConfiguration(extraConfiguration);

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService", RegistryOverlayImpl_Start_Test
                .createCfgServiceMock(domainConfiguration, containerConfiguration));
        try {
            registryOverlayImpl.start();
            final HazelcastClientProxy hzlClient = (HazelcastClientProxy) ReflectionHelper.getPrivateFieldValue(
                    RegistryOverlayImpl.class, registryOverlayImpl, "hzlClient");
            final Map<String, NearCacheConfig> nearCacheConfigs = hzlClient.getClientConfig().getNearCacheConfigMap();
            assertNotNull("No near cache configuration available", nearCacheConfigs);
            final Iterator<NearCacheConfig> itNearCacheConfigs = nearCacheConfigs.values().iterator();
            while (itNearCacheConfigs.hasNext()) {
                assertEquals(RegistryOverlayImpl.DEFAULT_MAP_CACHE_MAX_SIZE, itNearCacheConfigs.next().getMaxSize());
            }

        } finally {
            registryOverlayImpl.stop();
        }
    }

}
